@@ -417,10 +417,15 @@ sub new
 	} else {
 		print STDERR $response->status_line  if ($DEBUG);
 	}
+	unless ($html) {  #STEP 1 FAILED, INVALID URL, PUNT!
+		print STDERR "-!!!- COULD NOT FETCH URL=$url2fetch=\n"  if ($DEBUG);
+		return undef;
+	}
 	my $isHLSpage = ($html =~ /#EXTM3U/) ? 1 : 0;  #SEE AUDACIOUS ISSUE#1169 FOR EMBEDDED HLS IN HTML PAGE EXAMPLE USED FOR THIS CODE:
+	my $streamExts = join('|',@okStreams);
 	print STDERR "-1: HLS=$isHLSpage=\n----html=$html=\n"  if ($DEBUG > 1);
-	return undef  unless ($html && ($html =~ /\<\!DOCTYPE\s+(?:html|text)/i
-			|| $isHLSpage));  #STEP 1 FAILED, INVALID STATION URL, PUNT!
+	return undef  unless ($html =~ /\<\!DOCTYPE\s+(?:html|text)/i
+			|| ($isHLSpage && $streamExts =~ /(?:m3u8|any|all)/i));  #STEP 1 FAILED, INVALID STATION URL, PUNT!
 
 	print STDERR "-1: GOT SOME(<=1024 BYTES) HTML!\n"  if ($DEBUG);
 	if ($isHLSpage) {
@@ -434,86 +439,130 @@ sub new
 	} else {
 		print STDERR $response->status_line  if ($DEBUG);
 	}
-	if ($isHLSpage) {
+	unless ($html) {  #STEP 1 FAILED, INVALID URL, PUNT!
+		print STDERR "-!!!- COULD NOT FETCH URL=$url2fetch=\n"  if ($DEBUG);
+		return undef;
+	}
+
+	$self->{'_havehls'} = 0;
+	if ($isHLSpage && $url2fetch =~ /\.m3u8$/) {  #JWT:NOTE:MAY END UP NEEDING TO REMOVE TRAILING "$"!:
 		my $location = $response->header('location');
-		($baseURL = $location) =~ s#\/[^\/]+$##  if ($location =~ /^http/);
-		print STDERR "-!!!- HLS BASE URL=$baseURL=\n"  if ($DEBUG);
+		($baseURL = $url2fetch) =~ s#\/[^\/]+$##;
+		print STDERR "-!!!- WE'RE AN HLS PAGE!: HLS BASE URL=$baseURL= locn=$location\n"  if ($DEBUG);
+		my @lines = split(/\r?\n/, $html);
+		my @plentries = ();
+		my $firstTitle = '';
+		my $plidx = ($arglist =~ /\b\-?random\b/) ? 1 : 0;
+		my $line = 1;
+		(my $urlpath = $firstStream) =~ s#[^\/]+$##;
+		my $highestBW = 0;
+		my %streamsByBandwidth;
+		while ($line <= $#lines) {   #FIND HIGHEST BANDWIDTH STREAM (WITHIN ANY USER-SET BANDWIDTH):
+			if ($lines[$line] =~ /\s*\#EXT\-X\-STREAM\-INF\:(?:.*?)BANDWIDTH\=(\d+)/o) {
+				my $bw = $1;
+				if ($lines[$line] =~ /\,\s*URI\=\"([^\"]+)/o) { #SOMETIMES THEY STICK IT ON THE END OF CURRENT LINE!:
+					$lines[$line] = $1;
+				} else {
+					$line++;
+				}
+				if ($line <= $#lines) {
+					$bw =~ s/^\d*x//o;
+					if ($lines[$line] =~ m#\.m3u8#o
+							&& ($self->{'hls_bandwidth'} <= 0	|| $bw <= $self->{'hls_bandwidth'})) {
+						my $url = $lines[$line];
+						if ($lines[$line] =~ m#^https?\:\/\/#o) {
+							$streamsByBandwidth{$bw} = $lines[$line];
+						} elsif ($lines[$line] =~ m#^\/\/#o) {
+							my $protocol = $1  if ($url2fetch =~ /^(https?)/);
+							$streamsByBandwidth{$bw} = $protocol . ':' . $lines[$line];
+						} elsif ($lines[$line] =~ m#^\/#) {
+							$streamsByBandwidth{$bw} = $baseURL . $lines[$line];
+						} else {
+							$streamsByBandwidth{$bw} = $baseURL . '/' . $lines[$line];
+						}
+						$self->{'_havehls'} = 1;
+						print STDERR "----1($bw): found stream=$streamsByBandwidth{$bw}= bw=$bw=...\n"  if ($DEBUG);
+					}
+				}
+			}
+			$line++;
+		}
+		foreach $bw (sort {$b <=> $a} keys %streamsByBandwidth) {
+			push (@{$self->{'streams'}}, $streamsByBandwidth{$bw})
+					if (!$self->{'secure'} || $streamURL =~ /^https/o);
+		}
 	} else {
-		return undef  unless ($html);  #STEP 1 FAILED, INVALID STATION URL, PUNT!
-	}
-
-	my $location = $response->header('location');
-
-	$baseURL = $1  if ($html =~ m#\<BASE\s+HREF\=\"([^\"]+)#si);
-	$baseURL .= '/'  unless ($baseURL =~ m#\/$#o);
-	$self->{'title'} = $self->{'id'} || $url;
-	$self->{'description'} = $url2fetch;
-	$self->{'title'} = $1  if ($html =~ /\<TITLE>([^\<]+)/si);
-	print STDERR "-2: title=".$self->{'title'}."=\n"  if ($DEBUG);
-	$self->{'cnt'} = 0;  #LIMIT HOW MANY STREAMS TO CONSIDER - SOME PAGES (ie. $self->{'maxstreams'} CAN HAVE BAGILLIONS:
-	my $streams = '';
-	my $streamExts = join('|',@okStreams);
-	print STDERR "--EXTS=$streamExts=\n"  if ($DEBUG);
-	my $streamLimiter = 0;
-	while ($html =~ s#\<meta\s+([^\>]+)\>##iso) {
-		my $metatag = $1;
-		if ($metatag =~ m#\scontent\=\"([^\"]+)#io) {
-			my $content = $1;
-			$self->{'iconurl'} ||= $content  if ($metatag =~ /\:?image\"/o);
-			$self->{'title'} ||= $content  if ($metatag =~ /\:?title\"/o);
-			$self->{'description'} ||= $content  if ($metatag =~ /\:?description\"/o);
-			$self->{'artist'} ||= $content  if ($metatag =~ /\:?site_name\"/o);
-		}
-	}
-	while ($html =~ s#(https?\:\/\/|\.\.\/)?([^\s\'\"\:\<\>\[\]\{\}]+)\.($streamExts)##s
-			&& $streamLimiter <= $self->{'maxstreams'}) {
-		(my $one = $1 . $2) =~ s#\\\/#\/#gs;
-		my $ext = $3;
-		$one =~ s/\\u00([0-9A-Fa-f]{2})/chr(hex($1))/eg;
-		my $streamURL = $one.'.'.$ext;
-		print STDERR "--1: streamURL=$streamURL= baseURL=$baseURL=\n"  if ($DEBUG);
-		my $tmpbase = $baseURL;
-		while ($streamURL =~ s#\.\.\/##o) {
-			$tmpbase =~ s#[^\/]+\/$##o;
-		}
-		if ($streamURL =~ m#^\/\/#o) {  #STREAM URL STARTS WITH "//", ASSUME ABSOLUTE:
-			if ($baseURL =~ /http\:/io) {
-				$streamURL = 'http:' . $streamURL;
-			} else {
-				$streamURL = 'https:' . $streamURL;
+		print STDERR "--WE'RE A PAGE THAT MIGHT CONTAIN HLS:\n"  if ($DEBUG);
+		$baseURL = $1  if ($html =~ m#\<BASE\s+HREF\=\"([^\"]+)#si);
+		$baseURL .= '/'  unless ($baseURL =~ m#\/$#o);
+		$self->{'title'} = $self->{'id'} || $url;
+		$self->{'description'} = $url2fetch;
+		$self->{'title'} = $1  if ($html =~ /\<TITLE>([^\<]+)/si);
+		print STDERR "-2: title=".$self->{'title'}."=\n"  if ($DEBUG);
+		$self->{'cnt'} = 0;  #LIMIT HOW MANY STREAMS TO CONSIDER - SOME PAGES (ie. $self->{'maxstreams'} CAN HAVE BAGILLIONS:
+		my $streams = '';
+		print STDERR "--EXTS=$streamExts=\n"  if ($DEBUG);
+		my $streamLimiter = 0;
+		while ($html =~ s#\<meta\s+([^\>]+)\>##iso) {
+			my $metatag = $1;
+			if ($metatag =~ m#\scontent\=\"([^\"]+)#io) {
+				my $content = $1;
+				$self->{'iconurl'} ||= $content  if ($metatag =~ /\:?image\"/o);
+				$self->{'title'} ||= $content  if ($metatag =~ /\:?title\"/o);
+				$self->{'description'} ||= $content  if ($metatag =~ /\:?description\"/o);
+				$self->{'artist'} ||= $content  if ($metatag =~ /\:?site_name\"/o);
 			}
-			print STDERR "--2a: stream=$streamURL=\n"  if ($DEBUG);
-		} elsif ($streamURL =~ m#^\/#o) {  #STREAM URL STARTS WITH "/", ASSUME ABSOLUTE TO BASE PAGE URL ("TITLE"):
-			$tmpbase =~ s#\/$##o;
-			$streamURL = $tmpbase . $streamURL;
-			print STDERR "--2b: baseURL=$tmpbase= stream=$streamURL=\n"  if ($DEBUG);
-		} elsif ($streamURL !~ /^http/o) {  #NO PREFIX, ASSUME RELATIVE TO THE FETCHED URL ("LONG DESC."):
-			if ($isHLSpage) {
-				$tmpbase .= '/'  unless ($tmpbase =~ m#\/$#o);
+		}
+		while ($html =~ s#(https?\:\/\/|\.\.\/)?([^\s\'\"\:\<\>\[\]\{\}]+)\.($streamExts)##s
+				&& $streamLimiter <= $self->{'maxstreams'}) {
+			(my $one = $1 . $2) =~ s#\\\/#\/#gs;
+			my $ext = $3;
+			$one =~ s/\\u00([0-9A-Fa-f]{2})/chr(hex($1))/eg;
+			my $streamURL = $one.'.'.$ext;
+			print STDERR "--1: streamURL=$streamURL= baseURL=$baseURL=\n"  if ($DEBUG);
+			my $tmpbase = $baseURL;
+			while ($streamURL =~ s#\.\.\/##o) {
+				$tmpbase =~ s#[^\/]+\/$##o;
+			}
+			if ($streamURL =~ m#^\/\/#o) {  #STREAM URL STARTS WITH "//", ASSUME ABSOLUTE:
+				if ($baseURL =~ /http\:/io) {
+					$streamURL = 'http:' . $streamURL;
+				} else {
+					$streamURL = 'https:' . $streamURL;
+				}
+				print STDERR "--2a: stream=$streamURL=\n"  if ($DEBUG);
+			} elsif ($streamURL =~ m#^\/#o) {  #STREAM URL STARTS WITH "/", ASSUME ABSOLUTE TO BASE PAGE URL ("TITLE"):
+				$tmpbase =~ s#\/$##o;
 				$streamURL = $tmpbase . $streamURL;
-			} else {
-				$streamURL = $url2fetch . '/' . $streamURL;
+				print STDERR "--2b: baseURL=$tmpbase= stream=$streamURL=\n"  if ($DEBUG);
+			} elsif ($streamURL !~ /^http/o) {  #NO PREFIX, ASSUME RELATIVE TO THE FETCHED URL ("LONG DESC."):
+				if ($isHLSpage) {
+					$tmpbase .= '/'  unless ($tmpbase =~ m#\/$#o);
+					$streamURL = $tmpbase . $streamURL;
+				} else {
+					$streamURL = $url2fetch . '/' . $streamURL;
+				}
+				print STDERR "--2c: baseURL=$tmpbase= stream=$streamURL=\n"  if ($DEBUG);
+			} #OTHERWISE STREAM URL IS A FULL URL (NO CHANGE).
+			$streams .= "$ext=$streamURL|"  unless ($self->{'secure'} && $streamURL !~ /^https/o);
+			$streamLimiter++;
+		}
+		print STDERR "==streams=$streams=\n"  if ($DEBUG);
+		my $stindex = 0;
+		my $savestreams = $streams;
+		my %havestreams = ();
+		my ($one, $ext);
+		foreach my $streamtype (@okStreams) {
+			$streams = $savestreams;
+			while ($streams =~ /^([^\=]*)\=([^\|]+)/o) {
+				$ext = $1; $one = $2;
+				if ($ext =~ /^${streamtype}$/i && !defined($havestreams{"$ext|$one"})
+						&& (!$self->{'secure'} || $streamURL =~ /^https/o)) {
+					$self->{'streams'}->[$stindex++] = $one;
+					$havestreams{"$ext|$one"}++;
+				}
+				$streams =~ s/^[^\|]*\|//o;
 			}
-			print STDERR "--2c: baseURL=$tmpbase= stream=$streamURL=\n"  if ($DEBUG);
-		} #OTHERWISE STREAM URL IS A FULL URL (NO CHANGE).
-		$streams .= "$ext=$streamURL|"  unless ($self->{'secure'} && $streamURL !~ /^https/o);
-		$streamLimiter++;
-	}
-	print STDERR "==streams=$streams=\n"  if ($DEBUG);
-	my $stindex = 0;
-	my $savestreams = $streams;
-	my %havestreams = ();
-	my ($one, $ext);
-	foreach my $streamtype (@okStreams) {
-		$streams = $savestreams;
-		while ($streams =~ /^([^\=]*)\=([^\|]+)/o) {
-			$ext = $1; $one = $2;
-			if ($ext =~ /^${streamtype}$/i && !defined($havestreams{"$ext|$one"})
-					&& (!$self->{'secure'} || $streamURL =~ /^https/o)) {
-				$self->{'streams'}->[$stindex++] = $one;
-				$havestreams{"$ext|$one"}++;
-			}
-			$streams =~ s/^[^\|]*\|//o;
 		}
 	}
 	$self->{'cnt'} = scalar @{$self->{'streams'}};
@@ -547,4 +596,120 @@ sub new
 	return $self;
 }
 
+sub get
+{
+	my $self = shift;
+
+	return wantarray ? ($self->{'playlist'}) : $self->{'playlist'}  if (defined($_[0]) && $_[0] =~ /playlist/i);
+	return wantarray ? @{$self->{'streams'}} : ${$self->{'streams'}}[0];
+}
+
+sub getURL   #LIKE GET, BUT ONLY RETURN THE SINGLE ONE W/BEST BANDWIDTH AND RELIABILITY:
+{
+	my $self = shift;
+	my $arglist = (defined $_[0]) ? join('|',@_) : '';
+	my $idx = ($arglist =~ /\b\-?random\b/) ? int rand scalar @{$self->{'streams'}} : 0;
+	my $firstStream = ${$self->{'streams'}}[$idx];
+
+	return ''  unless (defined $firstStream);
+	return $firstStream  if ($self->{'_havehls'} == 1);
+
+	if (($arglist =~ /\b\-?nopls\b/ && $firstStream =~ /\.(pls|m3u)$/i)
+			|| (defined($self->{'hls_bandwidth'}) && $firstStream =~ /\.(m3u8)$/i)
+			|| ($arglist =~ /\b\-?noplaylists\b/ && $firstStream =~ /\.(pls|m3u8?)$/i)) {
+		my $plType = $1;
+		print STDERR "-getURL($idx): NOPLAYLISTS|BANDWIDTH and (".$firstStream.") TP=$plType=\n"  if ($DEBUG);
+		my $ua = LWP::UserAgent->new(@{$self->{'_userAgentOps'}});		
+		$ua->timeout($self->{'timeout'});
+		$ua->cookie_jar({});
+		$ua->env_proxy;
+		my $html = '';
+		my $response = $ua->get($firstStream);
+		if ($response->is_success) {
+			$html = $response->decoded_content;
+		} else {
+			print STDERR $response->status_line  if ($DEBUG);
+			my $no_wget = system('wget','-V');
+			unless ($no_wget) {
+				print STDERR "\n..trying wget...\n"  if ($DEBUG);
+				$html = `wget -t 2 -T 20 -O- -o /dev/null \"$firstStream\" 2>/dev/null `;
+			}
+		}
+		my @lines = split(/\r?\n/, $html);
+		my @plentries = ();
+		my $firstTitle = '';
+		my $plidx = ($arglist =~ /\b\-?random\b/) ? 1 : 0;
+		if ($plType =~ /pls/i) {  #PLS:
+			foreach my $line (@lines) {
+				if ($line =~ m#^\s*File\d+\=(.+)$#o) {
+					push (@plentries, $1);
+				} elsif ($line =~ m#^\s*Title\d+\=(.+)$#o) {
+					$firstTitle ||= $1;
+				}
+			}
+			$self->{'title'} ||= $firstTitle;
+			$self->{'title'} = HTML::Entities::decode_entities($self->{'title'});
+			$self->{'title'} = uri_unescape($self->{'title'});
+			print STDERR "-getURL(PLS): title=$firstTitle= pl_idx=$plidx=\n"  if ($DEBUG);
+			if ($plidx && $#plentries >= 0) {
+				$plidx = int rand scalar @plentries;
+			} else {
+				$plidx = 0;
+			}
+			$firstStream = $plentries[$plidx]
+					if (defined($plentries[$plidx]) && $plentries[$plidx]);
+		} elsif ($plType =~ /m3u8/i) {  #HLS?:
+			my $line = 1;
+			(my $urlpath = $firstStream) =~ s#[^\/]+$##;
+			my $highestBW = 0;
+			my $bestStream = '';
+			while ($line <= $#lines) {   #FIND HIGHEST BANDWIDTH STREAM (WITHIN ANY USER-SET BANDWIDTH):
+				if ($lines[$line] =~ /\s*\#EXT\-X\-STREAM\-INF\:(?:.*?)BANDWIDTH\=(\d+)/o) {
+					$line++;
+					if ($line <= $#lines) {
+						(my $bw = $1) =~ s/^\d*x//o;
+						if ($bw > $highestBW && $lines[$line] =~ m#\.m3u8#o
+								&& ($self->{'hls_bandwidth'} <= 0	|| $bw <= $self->{'hls_bandwidth'})) {
+							my $url = $lines[$line];
+							$highestBW = $bw;
+							if ($lines[$line] =~ m#^https?\:\/\/#o) {
+								$bestStream = $lines[$line];
+							} else {
+								$lines[$line] =~ s#^\/##o;
+								$bestStream = $urlpath . $lines[$line];
+							}
+							print STDERR "----2($bw): found stream=$bestStream= bw=$bw=...\n"  if ($DEBUG);
+						}
+					}
+				}
+				$line++;
+			}
+			$firstStream = $bestStream  if ($bestStream);
+			print STDERR "-getURL(m3u8/HLS) best=$bestStream=\n"  if ($DEBUG);
+		} else {  #m3u:
+			(my $urlpath = $firstStream) =~ s#[^\/]+$##;
+			foreach my $line (@lines) {
+				if ($line =~ m#^\s*([^\#].+)$#o) {
+					my $urlpart = $1;
+					$urlpart =~ s#^\s+##o;
+					$urlpart =~ s#^\/##o;
+					push (@plentries, ($urlpart =~ m#https?\:#) ? $urlpart : ($urlpath . $urlpart));
+					last  unless ($plidx);
+				}
+			}
+			if ($plidx && $#plentries >= 0) {
+				$plidx = int rand scalar @plentries;
+			} else {
+				$plidx = 0;
+			}
+			$firstStream = $plentries[$plidx]
+					if (defined($plentries[$plidx]) && $plentries[$plidx]);
+			print STDERR "-getURL(m3u): pl_idx=$plidx=\n"  if ($DEBUG);
+		}
+	}
+
+	print STDERR "-getURL returning stream=$firstStream=\n"  if ($DEBUG);
+	return $firstStream;
+}
+
 1
